@@ -26,13 +26,22 @@ import sys
 from typing import Callable, Container, Dict, FrozenSet, Mapping, Optional, Sequence, Set, Tuple
 from types import MappingProxyType
 
-from google.api_core import exceptions  # type: ignore
+from google.api_core import exceptions
+from google.api import http_pb2  # type: ignore
 from google.api import resource_pb2  # type: ignore
+from google.api import service_pb2  # type: ignore
+from google.cloud import extended_operations_pb2 as ex_ops_pb2  # type: ignore
+from google.gapic.metadata import gapic_metadata_pb2  # type: ignore
 from google.longrunning import operations_pb2  # type: ignore
-from google.protobuf import descriptor_pb2
-
+from google.iam.v1 import iam_policy_pb2  # type: ignore
+from google.cloud.location import locations_pb2  # type: ignore
+from google.protobuf import descriptor_pb2  # type: ignore
+from google.protobuf.json_format import MessageToJson
+from google.protobuf.json_format import ParseDict
+from google.protobuf.descriptor import ServiceDescriptor
 import grpc  # type: ignore
-
+from google.protobuf.descriptor_pb2 import MethodDescriptorProto
+from google.api import annotations_pb2  # type: ignore
 from gapic.schema import metadata
 from gapic.schema import wrappers
 from gapic.schema import naming as api_naming
@@ -43,6 +52,11 @@ from gapic.utils import to_snake_case
 from gapic.utils import RESERVED_NAMES
 
 
+TRANSPORT_GRPC = "grpc"
+TRANSPORT_GRPC_ASYNC = "grpc-async"
+TRANSPORT_REST = "rest"
+
+
 @dataclasses.dataclass(frozen=True)
 class Proto:
     """A representation of a particular proto file within an API."""
@@ -185,7 +199,7 @@ class Proto:
         answer = {
             t.ident.python_import
             for m in self.all_messages.values()
-            # Sanity check: We do make sure that we are not trying to have
+            # Quick check: We do make sure that we are not trying to have
             # a module import itself.
             for t in m.field_types if t.ident.python_import != self_reference
         }
@@ -219,6 +233,7 @@ class API:
     """
     naming: api_naming.Naming
     all_protos: Mapping[str, Proto]
+    service_yaml_config: service_pb2.Service
     subpackage_view: Tuple[str, ...] = dataclasses.field(default_factory=tuple)
 
     @classmethod
@@ -250,16 +265,28 @@ class API:
             file_descriptors,
         ), opts=opts)
 
-        def disambiguate_keyword_fname(
+        # "metadata", "retry", "timeout", and "request" are reserved words in client methods.
+        invalid_module_names = set(keyword.kwlist) | {
+            "metadata", "retry", "timeout", "request"}
+
+        def disambiguate_keyword_sanitize_fname(
                 full_path: str,
                 visited_names: Container[str]) -> str:
             path, fname = os.path.split(full_path)
             name, ext = os.path.splitext(fname)
-            if name in keyword.kwlist or full_path in visited_names:
+
+            # Replace `.` with `_` in the basename as
+            # `.` is not a valid character for modules names.
+            # See https://peps.python.org/pep-0008/#package-and-module-names
+            if "." in name:
+                name = name.replace(".", "_")
+                full_path = os.path.join(path, name + ext)
+
+            if name in invalid_module_names or full_path in visited_names:
                 name += "_"
                 full_path = os.path.join(path, name + ext)
                 if full_path in visited_names:
-                    return disambiguate_keyword_fname(full_path, visited_names)
+                    return disambiguate_keyword_sanitize_fname(full_path, visited_names)
 
             return full_path
 
@@ -278,7 +305,7 @@ class API:
         # load the services and methods with the full scope of types.
         pre_protos: Dict[str, Proto] = dict(prior_protos or {})
         for fd in file_descriptors:
-            fd.name = disambiguate_keyword_fname(fd.name, pre_protos)
+            fd.name = disambiguate_keyword_sanitize_fname(fd.name, pre_protos)
             pre_protos[fd.name] = Proto.build(
                 file_descriptor=fd,
                 file_to_generate=fd.package.startswith(package),
@@ -311,8 +338,14 @@ class API:
             for name, proto in pre_protos.items()
         }
 
+        # Parse the google.api.Service proto from the service_yaml data.
+        service_yaml_config = service_pb2.Service()
+        ParseDict(opts.service_yaml_config, service_yaml_config)
+
         # Done; return the API.
-        return cls(naming=naming, all_protos=protos)
+        return cls(naming=naming,
+                   all_protos=protos,
+                   service_yaml_config=service_yaml_config)
 
     @cached_property
     def enums(self) -> Mapping[str, wrappers.EnumType]:
@@ -367,6 +400,24 @@ class API:
                                     *[p.services for p in self.protos.values()],
                                     )
 
+    @cached_property
+    def http_options(self) -> Mapping[str, Sequence[wrappers.HttpRule]]:
+        """Return a map of API-wide http rules."""
+
+        def make_http_options(rule: http_pb2.HttpRule
+                              ) -> Sequence[wrappers.HttpRule]:
+            http_options = [rule] + list(rule.additional_bindings)
+            opt_gen = (wrappers.HttpRule.try_parse_http_rule(http_rule)
+                       for http_rule in http_options)
+            return [rule for rule in opt_gen if rule]
+
+        result: Mapping[str, Sequence[http_pb2.HttpRule]] = {
+            rule.selector: make_http_options(rule)
+            for rule in self.service_yaml_config.http.rules
+        }
+
+        return result
+
     @cached_property
     def subpackages(self) -> Mapping[str, 'API']:
         """Return a map of all subpackages, if any.
@@ -392,6 +443,46 @@ class API:
                                                       )
         return answer
 
+    def gapic_metadata(self, options: Options) -> gapic_metadata_pb2.GapicMetadata:
+        gm = gapic_metadata_pb2.GapicMetadata(
+            schema="1.0",
+            comment="This file maps proto services/RPCs to the corresponding library clients/methods",
+            language="python",
+            proto_package=self.naming.proto_package,
+            library_package=".".join(
+                self.naming.module_namespace +
+                    (self.naming.versioned_module_name,)
+            ),
+        )
+
+        for service in sorted(self.services.values(), key=lambda s: s.name):
+            service_desc = gm.services.get_or_create(service.name)
+
+            # At least one of "grpc" or "rest" is guaranteed to be present because
+            # of the way that Options instances are created.
+            # This assumes the options are generated by the class method factory.
+            transports = []
+            if "grpc" in options.transport:
+                transports.append((TRANSPORT_GRPC, service.client_name))
+                transports.append(
+                    (TRANSPORT_GRPC_ASYNC, service.async_client_name))
+
+            if "rest" in options.transport:
+                transports.append((TRANSPORT_REST, service.client_name))
+
+            methods = sorted(service.methods.values(), key=lambda m: m.name)
+            for tprt, client_name in transports:
+                transport = service_desc.clients.get_or_create(tprt)
+                transport.library_client = client_name
+                for method in methods:
+                    method_desc = transport.rpcs.get_or_create(method.name)
+                    method_desc.methods.append(to_snake_case(method.name))
+
+        return gm
+
+    def gapic_metadata_json(self, options: Options) -> str:
+        return MessageToJson(self.gapic_metadata(options), sort_keys=True)
+
     def requires_package(self, pkg: Tuple[str, ...]) -> bool:
         return any(
             message.ident.package == pkg
@@ -399,6 +490,99 @@ class API:
             for message in proto.all_messages.values()
         )
 
+    def get_custom_operation_service(self, method: "wrappers.Method") -> "wrappers.Service":
+        """Return the extended operation service that should be polled for progress
+        from a given initial method.
+
+        Precondition: `method` returns an Extended Operation type message
+        and has an `operation_polling_service` annotation.
+        """
+        if not method.output.is_extended_operation:
+            raise ValueError(
+                f"Method is not an extended operation LRO: {method.name}")
+
+        op_serv_name = self.naming.proto_package + "." + \
+            method.options.Extensions[ex_ops_pb2.operation_service]
+        op_serv = self.services.get(op_serv_name)
+        if not op_serv:
+            raise ValueError(
+                f"No such service: {op_serv_name}"
+            )
+
+        if not op_serv.operation_polling_method:
+            raise ValueError(
+                f"Service is not an extended operation operation service: {op_serv.name}")
+
+        return op_serv
+
+    @cached_property
+    def mixin_api_methods(self) -> Dict[str, MethodDescriptorProto]:
+        methods: Dict[str, MethodDescriptorProto] = {}
+        if self.has_location_mixin:
+            methods = {**methods, **
+                self._get_methods_from_service(locations_pb2)}
+        if not self._has_iam_overrides and self.has_iam_mixin:
+            methods = {**methods, **
+                self._get_methods_from_service(iam_policy_pb2)}
+        if self.has_operations_mixin:
+            methods = {**methods, **
+                self._get_methods_from_service(operations_pb2)}
+        return methods
+
+    @cached_property
+    def has_location_mixin(self) -> bool:
+        return len(list(filter(lambda api: api.name == "google.cloud.location.Locations", self.service_yaml_config.apis))) > 0
+
+    @cached_property
+    def has_iam_mixin(self) -> bool:
+        return len(list(filter(lambda api: api.name == "google.iam.v1.IAMPolicy", self.service_yaml_config.apis))) > 0
+
+    @cached_property
+    def has_operations_mixin(self) -> bool:
+        return len(list(filter(lambda api: api.name == "google.longrunning.Operations", self.service_yaml_config.apis))) > 0
+
+    @cached_property
+    def _has_iam_overrides(self) -> bool:
+        if not self.has_iam_mixin:
+            return False
+        iam_mixin_methods: Dict[str, MethodDescriptorProto] = self._get_methods_from_service(
+            iam_policy_pb2)
+        for (_, s) in self.services.items():
+            for m_name in iam_mixin_methods:
+                if m_name in s.methods:
+                    return True
+        return False
+
+    def _get_methods_from_service(self, service_pb) -> Dict[str, MethodDescriptorProto]:
+        services = service_pb.DESCRIPTOR.services_by_name
+        methods = {}
+        methods_to_generate = {}
+        for service_name in services:
+            service: ServiceDescriptor = services[service_name]
+            for method in service.methods:
+                fqn = "{}.{}.{}".format(
+                    service_pb.DESCRIPTOR.package, service.name, method.name)
+                methods[fqn] = method
+        for rule in self.service_yaml_config.http.rules:
+            if rule.selector in methods:
+                m = methods[rule.selector]
+                x = descriptor_pb2.MethodDescriptorProto()
+                m.CopyToProto(x)
+                x.options.Extensions[annotations_pb2.http].CopyFrom(rule)
+                methods_to_generate[x.name] = x
+        return methods_to_generate
+
+    def get_extended_operations_services(self, service) -> Set["wrappers.Service"]:
+        """Return a set of all the extended operation services used by the input service.
+
+        Precondition: `service` is NOT an extended operation service
+        """
+        return set(
+            self.get_custom_operation_service(m)
+            for m in service.methods.values()
+            if m.operation_service
+        )
+
 
 class _ProtoBuilder:
     """A "builder class" for Proto objects.
@@ -553,15 +737,25 @@ class _ProtoBuilder:
 
     @cached_property
     def api_enums(self) -> Mapping[str, wrappers.EnumType]:
-        return collections.ChainMap({}, self.proto_enums,
-                                    *[p.all_enums for p in self.prior_protos.values()],
-                                    )
+        return collections.ChainMap(
+            {},
+            self.proto_enums,
+            # This is actually fine from a typing perspective:
+            # we're agglutinating all the prior protos' enums, which are
+            # stored in maps. This is just a convenient way to expand it out.
+            *[p.all_enums for p in self.prior_protos.values()],  # type: ignore
+        )
 
     @cached_property
     def api_messages(self) -> Mapping[str, wrappers.MessageType]:
-        return collections.ChainMap({}, self.proto_messages,
-                                    *[p.all_messages for p in self.prior_protos.values()],
-                                    )
+        return collections.ChainMap(
+            {},
+            self.proto_messages,
+            # This is actually fine from a typing perspective:
+            # we're agglutinating all the prior protos' enums, which are
+            # stored in maps. This is just a convenient way to expand it out.
+            *[p.all_messages for p in self.prior_protos.values()],  # type: ignore
+        )
 
     def _load_children(self,
                        children: Sequence, loader: Callable, *,
@@ -770,6 +964,10 @@ class _ProtoBuilder:
         # If the output type is google.longrunning.Operation, we use
         # a specialized object in its place.
         if meth_pb.output_type.endswith('google.longrunning.Operation'):
+            if not meth_pb.options.HasExtension(operations_pb2.operation_info):
+                # This is not a long running operation even though it returns
+                # an Operation.
+                return None
             op = meth_pb.options.Extensions[operations_pb2.operation_info]
             if not op.response_type or not op.metadata_type:
                 raise TypeError(
@@ -786,6 +984,70 @@ class _ProtoBuilder:
 
         return lro
 
+    def _maybe_get_extended_lro(
+        self,
+        service_address: metadata.Address,
+        meth_pb: descriptor_pb2.MethodDescriptorProto,
+    ) -> Optional[wrappers.ExtendedOperationInfo]:
+        op_service_name = meth_pb.options.Extensions[ex_ops_pb2.operation_service]
+        if not op_service_name:
+            return None
+
+        # Manual lookups because services and methods haven't been loaded.
+        # Note: this assumes that the operation service lives in the same proto file.
+        # This is a reasonable assumption as of March '22, but it may (unlikely)
+        # change in the future.
+        op_service_pb = next(
+            (s for s in self.file_descriptor.service if s.name == op_service_name),
+            None,
+        )
+        if not op_service_pb:
+            raise ValueError(
+                f"Could not find custom operation service: {op_service_name}"
+            )
+
+        operation_polling_method_pb = next(
+            (
+                m
+                for m in op_service_pb.method
+                if m.options.Extensions[ex_ops_pb2.operation_polling_method]
+            ),
+            None,
+        )
+        if not operation_polling_method_pb:
+            raise ValueError(
+                f"Could not find operation polling method for custom operation service: {op_service_name}"
+            )
+
+        operation_request_key = service_address.resolve(
+            operation_polling_method_pb.input_type.lstrip(".")
+        )
+        operation_request_message = self.api_messages[operation_request_key]
+
+        operation_type = service_address.resolve(
+            operation_polling_method_pb.output_type.lstrip(".")
+        )
+        method_output_type = service_address.resolve(
+            meth_pb.output_type.lstrip(".")
+        )
+        if operation_type != method_output_type:
+            raise ValueError(
+                f"Inconsistent return types between extended lro method '{meth_pb.name}'"
+                f" and extended lro polling method '{operation_polling_method_pb.name}':"
+                f" '{method_output_type}' and '{operation_type}'"
+            )
+
+        operation_message = self.api_messages[operation_type]
+        if not operation_message.is_extended_operation:
+            raise ValueError(
+                f"Message is not an extended operation: {operation_type}"
+            )
+
+        return wrappers.ExtendedOperationInfo(
+            request_type=operation_request_message,
+            operation_type=operation_message,
+        )
+
     def _get_methods(self,
                      methods: Sequence[descriptor_pb2.MethodDescriptorProto],
                      service_address: metadata.Address, path: Tuple[int, ...],
@@ -816,6 +1078,10 @@ class _ProtoBuilder:
             answer[meth_pb.name] = wrappers.Method(
                 input=self.api_messages[meth_pb.input_type.lstrip('.')],
                 lro=self._maybe_get_lro(service_address, meth_pb),
+                extended_lro=self._maybe_get_extended_lro(
+                    service_address,
+                    meth_pb,
+                ),
                 method_pb=meth_pb,
                 meta=metadata.Metadata(
                     address=service_address.child(meth_pb.name, path + (i,)),
