@@ -27,7 +27,7 @@ with the things they describe for easy access in templates.
 """
 
 import dataclasses
-from typing import FrozenSet, Tuple
+from typing import FrozenSet, Tuple, Optional
 
 from google.protobuf import descriptor_pb2
 
@@ -36,25 +36,30 @@ from gapic.schema import naming
 from gapic.utils import cached_property
 from gapic.utils import RESERVED_NAMES
 
+# This class is a minor hack to optimize Address's __eq__ method.
+
 
 @dataclasses.dataclass(frozen=True)
-class Address:
+class BaseAddress:
     name: str = ''
     module: str = ''
     module_path: Tuple[int, ...] = dataclasses.field(default_factory=tuple)
     package: Tuple[str, ...] = dataclasses.field(default_factory=tuple)
     parent: Tuple[str, ...] = dataclasses.field(default_factory=tuple)
+
+
+@dataclasses.dataclass(frozen=True)
+class Address(BaseAddress):
     api_naming: naming.Naming = dataclasses.field(
         default_factory=naming.NewNaming,
     )
     collisions: FrozenSet[str] = dataclasses.field(default_factory=frozenset)
 
-    def __post_init__(self):
-        super().__setattr__("collisions", self.collisions | RESERVED_NAMES)
-
     def __eq__(self, other) -> bool:
-        return all([getattr(self, i) == getattr(other, i) for i
-                    in ('name', 'module', 'module_path', 'package', 'parent')])
+        # We don't want to use api_naming or collisions to determine equality,
+        # so defer to the parent class's eq method.
+        # This is an fairly important optimization for large APIs.
+        return super().__eq__(other)
 
     def __hash__(self):
         # Do NOT include collisions; they are not relevant.
@@ -77,12 +82,19 @@ class Address:
         """
         # Most (but not all) types are in a module.
         if self.module:
+            module_name = self.module
+
             # If collisions are registered and conflict with our module,
             # use the module alias instead.
-            module_name = self.module
             if self.module_alias:
                 module_name = self.module_alias
 
+            # This module is from a different proto package
+            # Most commonly happens for a common proto
+            # https://pypi.org/project/googleapis-common-protos/
+            if not self.proto_package.startswith(self.api_naming.proto_package):
+                module_name = f'{self.module}_pb2'
+
             # Return the dot-separated Python identifier.
             return '.'.join((module_name,) + self.parent + (self.name,))
 
@@ -90,7 +102,8 @@ class Address:
         # Return the Python identifier.
         return '.'.join(self.parent + (self.name,))
 
-    def __repr__(self) -> str:
+    @cached_property
+    def __cached_string_repr(self):
         return "({})".format(
             ", ".join(
                 (
@@ -104,6 +117,9 @@ class Address:
             )
         )
 
+    def __repr__(self) -> str:
+        return self.__cached_string_repr
+
     @property
     def module_alias(self) -> str:
         """Return an appropriate module alias if necessary.
@@ -114,12 +130,14 @@ class Address:
         while still providing names that are fundamentally readable
         to users (albeit looking auto-generated).
         """
-        if self.module in self.collisions:
+        # This is a minor optimization to prevent constructing a temporary set.
+        if self.module in self.collisions or self.module in RESERVED_NAMES:
             return '_'.join(
                 (
                     ''.join(
-                        i[0]
+                        partial_name[0]
                         for i in self.package
+                        for partial_name in i.split("_")
                         if i != self.api_naming.version
                     ),
                     self.module,
@@ -170,15 +188,27 @@ class Address:
         return imp.Import(
             package=self.package,
             module=f'{self.module}_pb2',
-            alias=self.module_alias if self.module_alias else self.module,
         )
 
     @property
     def sphinx(self) -> str:
         """Return the Sphinx identifier for this type."""
-        if self.module:
-            return f'~.{self}'
-        return self.name
+
+        if not self.api_naming:
+            if self.package:
+                return '.'.join(self.package + (self.module, self.name))
+            else:
+                return str(self)
+
+        # Check if this is a generated type
+        # Use the original module name rather than the module_alias
+        if self.proto_package.startswith(self.api_naming.proto_package):
+            return '.'.join(self.api_naming.module_namespace + (
+                self.api_naming.versioned_module_name,
+            ) + self.subpackage + ('types',) + self.parent + (self.name, ))
+
+        # Anything left is a standard _pb2 type
+        return f'{self.proto_package}.{self.module}_pb2.{self.name}'
 
     @property
     def subpackage(self) -> Tuple[str, ...]:
@@ -283,7 +313,11 @@ class Address:
         ``Address`` object aliases module names to avoid naming collisions in
         the file being written.
         """
-        return dataclasses.replace(self, collisions=frozenset(collisions))
+        return (
+            dataclasses.replace(self, collisions=collisions)
+            if collisions and collisions != self.collisions
+            else self
+        )
 
 
 @dataclasses.dataclass(frozen=True)
@@ -321,21 +355,26 @@ class Metadata:
         return dataclasses.replace(
             self,
             address=self.address.with_context(collisions=collisions),
-        )
+        ) if collisions and collisions != self.address.collisions else self
 
 
 @dataclasses.dataclass(frozen=True)
 class FieldIdentifier:
     ident: Address
     repeated: bool
+    mapping: Optional[tuple] = None
 
     def __str__(self) -> str:
+        if self.mapping:
+            return f'Mapping[{self.mapping[0].ident}, {self.mapping[1].ident}]'
         if self.repeated:
             return f'Sequence[{self.ident}]'
         return str(self.ident)
 
     @property
     def sphinx(self) -> str:
+        if self.mapping:
+            return f'Mapping[{self.mapping[0].ident.sphinx}, {self.mapping[1].ident.sphinx}]'
         if self.repeated:
             return f'Sequence[{self.ident.sphinx}]'
         return self.ident.sphinx
