@@ -21,7 +21,12 @@ import keyword
 # They are explicitly allowed message, module, and field names.
 RESERVED_NAMES = frozenset(
     itertools.chain(
+        # We CANNOT make exceptions for keywords.
         keyword.kwlist,
-        set(dir(builtins)) - {"filter", "map", "id", "property"},
+        # We make SOME exceptions for certain names that collide with builtins.
+        set(dir(builtins)) - {"filter", "map", "id", "input", "property"},
+        # "mapping" and "ignore_unknown_fields" have special uses
+        # in the constructor of proto.Message
+        {"mapping", "ignore_unknown_fields"},
     )
 )
