@@ -33,7 +33,7 @@ def test_address_str_with_context():
         package=('foo', 'bar'),
         module='baz',
         name='Bacon',
-    ).with_context(collisions={'baz'})
+    ).with_context(collisions=frozenset({'baz'}))
     assert str(addr) == 'fb_baz.Bacon'
 
 
@@ -43,6 +43,23 @@ def test_address_str_parent():
     assert str(addr) == 'baz.spam.eggs.Bacon'
 
 
+def test_address_str_different_proto_package():
+    addr = metadata.Address(package=('google', 'iam', 'v1'), module='options', name='GetPolicyOptions',
+        api_naming=naming.NewNaming(proto_package='foo.bar.baz.v1'))
+    assert str(addr) == 'options_pb2.GetPolicyOptions'
+
+
+def test_address_str_different_proto_package_with_collision():
+    addr = metadata.Address(
+        package=('google', 'rpc'),
+        module='status',
+        name='Status',
+        api_naming=naming.NewNaming(proto_package='foo.bar.baz.v1')
+    ).with_context(collisions=frozenset({'status'}))
+    # the module alias should be ignored for _pb2 types
+    assert str(addr) == 'status_pb2.Status'
+
+
 def test_address_proto():
     addr = metadata.Address(package=('foo', 'bar'), module='baz', name='Bacon')
     assert addr.proto == 'foo.bar.Bacon'
@@ -184,6 +201,14 @@ def test_address_name_builtin_keyword():
     )
     assert addr_kword.module_alias == "gp_class"
 
+    addr_kword = metadata.Address(
+        name="Class",
+        module="class",
+        package=("google", "appengine_admin"),
+        api_naming=naming.NewNaming(proto_package="foo.bar.baz.v1"),
+    )
+    assert addr_kword.module_alias == "gaa_class"
+
 
 def test_doc_nothing():
     meta = metadata.Metadata()
